package Finance::Bank::SuomenVerkkomaksut;
BEGIN {
  $Finance::Bank::SuomenVerkkomaksut::AUTHORITY = 'cpan:OKKO';
}
$Finance::Bank::SuomenVerkkomaksut::VERSION = '0.009';
use utf8;
use strict;

=encoding utf-8
=cut

# ABSTRACT: DEPRECATED. Use Finance::Bank::Paytrail instead.

=head1 NAME

Finance::Bank::SuomenVerkkomaksut - DEPRECATED. Use Finance::Bank::Paytrail instead.

=cut

1;
